"""
This module contains functionality for reading "System overview" data
from the PacketLogic system.

Do NOT import this module directly. The L{PLConnection
<packetlogic2.PLConnection>} object should be used, e.g:

    >>> import packetlogic2
    >>> pl = packetlogic2.connect("192.168.1.25", "admin", "pldemo00")
    >>> so = pl.SystemOverview()

@var __revision__: Revision identifier.
"""

__revision__ = '%K%'

from resource import Resource
from thisversion import PLDB

import plutil

_Prop_SystemOverviewSystem = plutil.Prop('systemid', 'name', 'host', 'lastupdate', 'firstupdate', 'distversion', 'drdlrevision', values = plutil.RoList)
from packetlogic2.exceptions import PLDBUnsupportedInFirmware

class System(object):
	""" Information about a System in the SystemOverview.
	
	Returned in a list by L{SystemOverview.system_list}.
	
	@type systemid: C{str}
	@type name: C{str}
	@type host: C{str}
	@type lastupdate: C{str}
	@type firstupdate: C{str}
	@type distversion: C{str}
	@type drdlrevision: C{str}
	@type values: read-only C{list} of C{tuple}s
	
	@cvar __slots__: For internal use only.
	"""
	__slots__ = ('_prop', '_pldb')
	
	def __init__(self, _pldb=None, **props):
		self._pldb = _pldb
		self._prop = _Prop_SystemOverviewSystem(**props)

	def __repr__(self):
		return "<%s %s @ %s '%s'>" % (self.__class__.__name__, self._prop.systemid, self._prop.host, self._prop.name)

	systemid = property(lambda self: self._prop.systemid, doc="SystemID (MachineID).")
	name = property(lambda self: self._prop.name, doc="Name given to identify this system.")
	host = property(lambda self: self._prop.host, doc="Host name or IP address.")
	lastupdate = property(lambda self: self._prop.lastupdate, doc="ISO Date/time when last updated.")
	firstupdate = property(lambda self: self._prop.firstupdate, doc="ISO Date/time when first updated.")
	distversion = property(lambda self: self._prop.distversion, doc="Version of distribution/firmware.")
	drdlrevision = property(lambda self: self._prop.drdlrevision, doc="DRDL Revision.")
	values = property(lambda self: self._prop.values, doc="Read-only C{list} of two-item C{tuple} values.")

# At the moment there is just one signel flag, but since it is a bitmap, we prepare for the future with a Flags object.
_Prop_TemplateFlags = plutil.Prop(summary=bool)

class TemplateFlags(plutil.BaseFlags):
	__slots__ = ('_prop')
	_Prop_class = _Prop_TemplateFlags
	
	def _set_flags_other_type(self, flags):
		if isinstance(flags, (int, long)):
			self._prop.update(summary=flags & PLDB.SYSTEM_OVERVIEW_FLAGS_SUMMARY)
		else:
			plutil.BaseFlags._set_flags_other_type(self, flags)
		
	#def __int__(self):
	#	return (self._prop.summary * PLDB.SYSTEM_OVERVIEW_FLAGS_SUMMARY)
	
	summary = property(lambda self: self._prop.summary)

_Prop_SystemOverviewTemplate = plutil.Prop('templateid', 'name', 'type', 'vtype', 'zone', 'id', 'ttype', 'descstr', flags=TemplateFlags)

class Template(object):
	""" Information about a Template in the SystemOverview.
	
	Returned in a list by L{SystemOverview.template_list}.
	
	@type templateid: C{int}
	@type name: C{str}
	@type type: C{int}
	@type flags: L{Flags <plutil.Flags>} object
	@type vtype: C{int}
	@type zone: C{int}
	@type id: C{int}
	@type ttype: C{int}
	
	@cvar __slots__: For internal use only.
	"""
	__slots__ = ('_prop', '_pldb')
	
	def __init__(self, _pldb=None, **props):
		self._pldb = _pldb
		self._prop = _Prop_SystemOverviewTemplate(**props)

	def __repr__(self):
		return "<%s #%d '%s'>" % (self.__class__.__name__, self._prop.templateid, self._prop.name)

	templateid = property(lambda self: self._prop.templateid, doc="Internal ID, used in L{SystemOverview.system_graph} call")
	name = property(lambda self: self._prop.name, doc="Name of the template.")
	type = property(lambda self: self._prop.type, doc="one of the TYPE_ constants")
	flags = property(lambda self: self._prop.flags, doc="Boolean flag(s): summary")
	vtype = property(lambda self: self._prop.vtype, doc="one of the VTYPE_ constants")
	zone = property(lambda self: self._prop.zone, doc="Zone ID, one of the SYSDIAG_ZONE_ constants")
	id = property(lambda self: self._prop.id, doc="Values internal ID within the zone.")
	ttype = property(lambda self: self._prop.ttype, doc="one of the TEMPLATE_TYPE_ constants")
	descstr = property(lambda self: self._prop.descstr, doc="String description of value (usually OID)")

_NONEXISTANT_TYPE = object()

class SystemOverview(Resource):
	""" This is the resource for System Overview.

	@cvar TYPE_STRING: Type of the value is a string.
	@cvar TYPE_INT: Type of the value is an integer.
	@cvar TYPE_RATE: Type of the value is an integer representing a rate (per second).
	@cvar TYPE_BANDWIDTH: Type of the value is an integer representing bandwidth (bps)
	@cvar TYPE_TRANSFER: Type of the value is an integer representing bytes transfered.
	@cvar TYPE_DURATION: Type of the value is a time duration.
	@cvar TYPE_TIMESTAMP: Type of the value is a timestamp.
	@cvar TYPE_BITMASK: Type of the value is an integer representing a bitmask.
	@cvar TYPE_INT_K: Type of the value is an integer measuring thousands.
	
	@cvar VTYPE_TOTAL: Value is a total since system restart.
	@cvar VTYPE_RATE: Value is a speed or rate.
	@cvar VTYPE_MAX: Value is the maximum reached since system restart.
	
	@cvar TEMPLATE_TYPE_SYSDIAG: Value is from System Diagnostics.
	@cvar TEMPLATE_TYPE_CHANNEL: Value is from Channel Statistics.
	
	@cvar SYSDIAG_ZONE_: All Stats Zones enumerated.
	
	"""
	
	TYPE_STRING = PLDB.SYSTEM_OVERVIEW_TYPE_STRING
	TYPE_INT = PLDB.SYSTEM_OVERVIEW_TYPE_INT
	TYPE_RATE = PLDB.SYSTEM_OVERVIEW_TYPE_RATE
	TYPE_BANDWIDTH = PLDB.SYSTEM_OVERVIEW_TYPE_BANDWIDTH
	TYPE_TRANSFER = PLDB.SYSTEM_OVERVIEW_TYPE_TRANSFER
	TYPE_DURATION = PLDB.SYSTEM_OVERVIEW_TYPE_DURATION
	TYPE_TIMESTAMP = PLDB.SYSTEM_OVERVIEW_TYPE_TIMESTAMP
	TYPE_BITMASK = PLDB.SYSTEM_OVERVIEW_TYPE_BITMASK
	TYPE_INT_K = PLDB.SYSTEM_OVERVIEW_TYPE_INT_K

	VTYPE_TOTAL = PLDB.SYSTEM_OVERVIEW_VTYPE_TOTAL
	VTYPE_RATE = PLDB.SYSTEM_OVERVIEW_VTYPE_RATE
	VTYPE_MAX = PLDB.SYSTEM_OVERVIEW_VTYPE_MAX
	TEMPLATE_TYPE_SYSDIAG = PLDB.SYSTEM_OVERVIEW_TEMPLATE_TYPE_SYSDIAG
	TEMPLATE_TYPE_STATS = plutil.deprecatedalias('TEMPLATE_TYPE_STATS', 'TEMPLATE_TYPE_SYSDIAG')
	TEMPLATE_TYPE_CHANNEL = PLDB.SYSTEM_OVERVIEW_TEMPLATE_TYPE_CHANNEL

	SYSDIAG_ZONE_INTERFACE = PLDB.SYSDIAG_ZONE_INTERFACE
	SYSDIAG_ZONE_MEMORY = PLDB.SYSDIAG_ZONE_MEMORY
	SYSDIAG_ZONE_PACKETPROCESSING = PLDB.SYSDIAG_ZONE_PACKETPROCESSING
	SYSDIAG_ZONE_IPV4 = PLDB.SYSDIAG_ZONE_IPV4
	SYSDIAG_ZONE_HOST = PLDB.SYSDIAG_ZONE_HOST
	SYSDIAG_ZONE_TCPV4 = PLDB.SYSDIAG_ZONE_TCPV4
	SYSDIAG_ZONE_CONNECTION = PLDB.SYSDIAG_ZONE_CONNECTION
	SYSDIAG_ZONE_RULESET = PLDB.SYSDIAG_ZONE_RULESET
	SYSDIAG_ZONE_FILTERING = PLDB.SYSDIAG_ZONE_FILTERING
	SYSDIAG_ZONE_SHAPING = PLDB.SYSDIAG_ZONE_SHAPING
	SYSDIAG_ZONE_LIST = PLDB.SYSDIAG_ZONE_LIST
	SYSDIAG_ZONE_HASH = PLDB.SYSDIAG_ZONE_HASH
	SYSDIAG_ZONE_PPPOE = PLDB.SYSDIAG_ZONE_PPPOE
	SYSDIAG_ZONE_GENERAL = PLDB.SYSDIAG_ZONE_GENERAL
	SYSDIAG_ZONE_BGP = PLDB.SYSDIAG_ZONE_BGP
	SYSDIAG_ZONE_USER_MEMORY = PLDB.SYSDIAG_ZONE_USER_MEMORY
	SYSDIAG_ZONE_USER_LIST = PLDB.SYSDIAG_ZONE_USER_LIST
	SYSDIAG_ZONE_USER_HASH = PLDB.SYSDIAG_ZONE_USER_HASH
	SYSDIAG_ZONE_USER_REAPER_CMD = PLDB.SYSDIAG_ZONE_USER_REAPER_CMD
	SYSDIAG_ZONE_DRDL = PLDB.SYSDIAG_ZONE_DRDL
	SYSDIAG_ZONE_ETHERNET = PLDB.SYSDIAG_ZONE_ETHERNET

	STATS_ZONE_INTERFACE = plutil.deprecatedalias('STATS_ZONE_INTERFACE', 'SYSDIAG_ZONE_INTERFACE')
	STATS_ZONE_MEMORY = plutil.deprecatedalias('STATS_ZONE_MEMORY', 'SYSDIAG_ZONE_MEMORY')
	STATS_ZONE_LINKLEVEL = plutil.deprecatedalias('STATS_ZONE_LINKLEVEL', 'SYSDIAG_ZONE_LINKLEVEL')
	STATS_ZONE_IPV4 = plutil.deprecatedalias('STATS_ZONE_IPV4', 'SYSDIAG_ZONE_IPV4')
	STATS_ZONE_HOST = plutil.deprecatedalias('STATS_ZONE_HOST', 'SYSDIAG_ZONE_HOST')
	STATS_ZONE_TCPV4 = plutil.deprecatedalias('STATS_ZONE_TCPV4', 'SYSDIAG_ZONE_TCPV4')
	STATS_ZONE_CONNECTION = plutil.deprecatedalias('STATS_ZONE_CONNECTION', 'SYSDIAG_ZONE_CONNECTION')
	STATS_ZONE_RULESET = plutil.deprecatedalias('STATS_ZONE_RULESET', 'SYSDIAG_ZONE_RULESET')
	STATS_ZONE_FW = plutil.deprecatedalias('STATS_ZONE_FW', 'SYSDIAG_ZONE_FILTERING')
	STATS_ZONE_SHAPING = plutil.deprecatedalias('STATS_ZONE_SHAPING', 'SYSDIAG_ZONE_SHAPING')
	STATS_ZONE_LIST = plutil.deprecatedalias('STATS_ZONE_LIST', 'SYSDIAG_ZONE_LIST')
	STATS_ZONE_HASH = plutil.deprecatedalias('STATS_ZONE_HASH', 'SYSDIAG_ZONE_HASH')
	STATS_ZONE_PPPOE = plutil.deprecatedalias('STATS_ZONE_PPPOE', 'SYSDIAG_ZONE_PPPOE')
	STATS_ZONE_GENERAL = plutil.deprecatedalias('STATS_ZONE_GENERAL', 'SYSDIAG_ZONE_GENERAL')
	STATS_ZONE_USER_BGP = plutil.deprecatedalias('STATS_ZONE_USER_BGP', 'SYSDIAG_ZONE_BGP')
	STATS_ZONE_USER_MEMORY = plutil.deprecatedalias('STATS_ZONE_USER_MEMORY', 'SYSDIAG_ZONE_USER_MEMORY')
	STATS_ZONE_USER_LIST = plutil.deprecatedalias('STATS_ZONE_USER_LIST', 'SYSDIAG_ZONE_USER_LIST')
	STATS_ZONE_USER_HASH = plutil.deprecatedalias('STATS_ZONE_USER_HASH', 'SYSDIAG_ZONE_USER_HASH')
	STATS_ZONE_USER_REAPER_CMD = plutil.deprecatedalias('STATS_ZONE_USER_REAPER_CMD', 'SYSDIAG_ZONE_USER_REAPER_CMD')
	STATS_ZONE_DRDL = plutil.deprecatedalias('STATS_ZONE_DRDL', 'SYSDIAG_ZONE_DRDL')
	STATS_ZONE_ETHERNET = plutil.deprecatedalias('STATS_ZONE_ETHERNET', 'SYSDIAG_ZONE_ETHERNET')

	def _resource(self):
		return PLDB.RESOURCE_SYSTEM_OVERVIEW

	def template_list(self):
		""" Return templates used by systems overview.
		
		@returns: C{list} of L{Template} objects.
		"""
		
		return plutil._do(self, self.session.system_overview_template_list(),
				  plutil._ListCB(self, self.session.system_overview_template_list_handle, Template))

	def system_list(self):
		""" Return systems defined in systems overview.
		
		@returns: C{list} of L{System} objects.
		"""
		
		return plutil._do(self, self.session.system_overview_list(),
				  plutil._ListCB(self, self.session.system_overview_list_handle, System))

	def system_graph(self, templateid, systemid=None):
		""" Return a C{list} of graph data for the specified system.
		Each value represent one minute, and there are 1439 values (24 hours
		except the minute in progress). Each value is an C{int} with the
		value, or C{False} if data was missing at that point.

		@type templateid: C{int}
		@param templateid: The templateid to use, see L{template_list}.
		
		@type systemid: C{str}
		@param systemid: The systemid for which to fetch data.
				 See L{system_list}. Defaults to the connected system.

		@returns: C{list} of C{int} or {False}

		@raises ValueError: if no data can be read for the specified criteria.
		"""
		if systemid is None:
			systemid = self.systemid
			
		lst = plutil._do(self, self.session.system_overview_system_graph(systemid, templateid),
				  plutil._ListCB(self, self.session.system_overview_system_graph_handle, False, False))

		# emulate old behavior without timestamps
		lst = [v[0] for v in lst]

		if not [v for v in lst if v is not False]:
			# This is (most likely) an invalid systemid or templateid.
			raise ValueError, "Can't fetch information for the specified systemid and templateid"

		return lst

	def _system_announce(self, systemid, name, distversion, drdlrevision, configmd5, model):
		"""
		Only for internal use.
		"""
		return plutil._do(self, self.session.system_overview_announce(systemid, name, distversion, drdlrevision, configmd5, model),
		                  plutil._AddCB(self))

	def _system_delete(self, systemid):
		"""
		Only for internal use.
		"""
		return plutil._do(self, self.session.system_overview_system_delete(systemid), plutil._AddCB(self))

	def sysinfo_systems_list(self):
		"""Return systems having reported sysinfo data

		@returns: C{list} of C{str}.

		@raises PLDBUnsupportedInFirmware: If used on firmware without support for sysinfo.

		@attention: sysinfo requires v14.0 firmware or newer.
		"""
		return plutil._do(self, self.session.system_overview_sysinfo_list_systems(),
				  plutil._SimpleListCB(self, self.session.system_overview_sysinfo_list_systems_handle, None))

	def _sysinfo_add(self, system, data):
		"""
		Only for internal use.
		data should be (key, value, children), e.g:

		("root", "", [("chassis", "", [("slot", "1", [("boottime", "1000", []),
		                                              ("version", "1234", []),
		                                              ("status", "ok", [])]),
		                               ("slot", "2", [("boottime", "1337", []),
		                                              ("version", "1234", []),
		                                              ("status", "ok", [])])])])

		@raises PLDBUnsupportedInFirmware: If used on firmware without support for sysinfo.

		@attention: sysinfo requires v14.0 firmware or newer.
		"""
		return plutil._do(self, self.session.system_overview_sysinfo_add_system(system, data),
				  plutil._AddCB(self))

	def sysinfo_get(self, systemid):
		"""Get the sysinfo tree reported for specified system.

		The returned data is in the format (key, value,
		children), where children is a list with same
		format. e.g:

		("root", "", [("chassis", "", [("slot", "1", [("boottime", "1000", []),
		                                              ("version", "1234", []),
		                                              ("status", "ok", [])]),
		                               ("slot", "2", [("boottime", "1337", []),
		                                              ("version", "1234", []),
		                                              ("status", "ok", [])])])])



		@type systemid: C{str}
		@param systemid: The system to retrieve data for. See also L{sysinfo_systems_list}.

		@returns: C{tuple} (key, value, children)

		@raises PLDBUnsupportedInFirmware: If used on firmware without support for sysinfo.

		@attention: sysinfo requires v14.0 firmware or newer.
		"""
		return plutil._do(self, self.session.system_overview_sysinfo_get_system(systemid),
				  plutil._AddCB(self, self.session.system_overview_sysinfo_get_system_handle))
