"""
This module contains PropertyObject implementation.

There is no need to import this module directly. Use the methods of
the L{Ruleset object <ruleset.Ruleset>} to create and manipulate
PropertyObjects.

    >>> o = r.object_add("/PropertyObjects/Test")
    >>> o.add("Filename=/index.html")
    >>> print o.items
    ['Filename=/index.html']

@var __revision__: Revision identifier.
"""

__revision__ = '%K%'

import plobject
from thisversion import PLDB
import plutil
from plutil import verify_ip

from thisversion import PL

#Max size of the value field in database. VARCHAR(255)
VALUE_MAXLEN = 255

class PropertyObject(plobject.PLObject):
	"""
	PropertyObject Implementation

	Object path: /PropertyObjects

	@cvar Item: PropertyObject Item implementation.
	"""

	__slots__ = ('_prop', '_pldb', '_callback', '_item_added')
	
	base_path = plutil.roConst("/PropertyObjects", "Only the type portion of the path")
	type = plutil.roConst(PLDB.OBJECT_TYPE_PROPERTYOBJECT, "Number representing the type of object")

	def _create_item(self, value):
		if "=" not in value:
			raise ValueError("Format error on string should be prop=value")

		property, val = value.split("=", 1)
		
		item = self.Item(self, property=property, value=val)

		if len(val) > VALUE_MAXLEN:
			raise ValueError ("Property value is too long. Max %u chars." % VALUE_MAXLEN)

		if item._prop.property not in self._pldb.services_prop_list():
			raise ValueError("Unknown property '%s'. Check ruleset.services_prop_list()." % item._prop.property)
		
		return item

	def add(self, value):
		"""
		Add a PropertyObject Item to this PropertyObject

		@type value: C{str}
		@param value: A string that describes the value of the PropertyObject item.
			Should be formatted as I{property=value}

			Wildcards are allowed in value:

				- * matches anything
				- ? matches one arbitrary character

			Wildcard-characters can be escaped using \ to match the literal character.

		"""
		plobject.PLObject.add(self, value)

	def value2cipher(key, value):
		""" Encrypts the property value. Can for example be used when filtering
		sensitive URLs.

		You need to add the key to Realtime. Example:

		>>> pl = packetlogic2.connect(...)
		>>> rt = pl.Realtime()
		>>> rt.keyvault.set(1, "password")

		This will add the key "password" as key index 1.

		You need to set the encrypted attribute to the key index it was encrypted
		with on the PropertyObject holding the encrypted PropertyObject Items:

		>>> rs = pl.Ruleset()
		>>> o = rs.object_find("/PropertyObject/EncryptedExample"
		>>> o.attributes['encrypted'] = "1"

		Example where an url is encrypted and added:

		>>> url = "http://www.example.com/page.html"
		>>> cipher = o.value2cipher("password", url)
		>>> o.add("URL=" + cipher)
		"""
		plain_maxlen = PL.strcrypt_decrypt_maxlen(VALUE_MAXLEN)
		return PL.strcrypt_plain2cipher(key, value, plain_maxlen)

	# We are supporting Python 2.3 and can't use @staticmethod decorator
	value2cipher = staticmethod(value2cipher)

	def cipher2value(key, cipher):
		""" Inverse of value2cipher """
		return PL.strcrypt_cipher2plain(key, cipher)

	# We are supporting Python 2.3 and can't use @staticmethod decorator
	cipher2value = staticmethod(cipher2value)

	class Item(plobject.PLObject.Item):
		"""
		PropertyObject Item implementation.
		Should not be instantiated, use L{PropertyObject.add} instead.

		@type property: str
		@type value: str
		"""
		__slots__ = ('_prop', '_parent')
		_Prop = plutil.Prop(property=str, value=str, *plobject._item_props)

		def __str__(self):
			return "%s=%s" % (self._prop.property, self._prop.value)

		value = property(lambda self: self._prop.value, doc="What to match in the property")

		# Make sure to put this one last, since now, unfortunately, property is a reserved name that gets overwritten... :/
		
		property = property(lambda self: self._prop.property, doc="The property matching")
